<?php
// Version: 2.0 RC4; index

/*	This template is, perhaps, the most important template in the theme. It
	contains the main template layer that displays the header and footer of
	the forum, namely with main_above and main_below. It also contains the
	menu sub template, which appropriately displays the menu; the init sub
	template, which is there to set the theme up; (init can be missing.) and
	the linktree sub template, which sorts out the link tree.

	The init sub template should load any data and set any hardcoded options.

	The main_above sub template is what is shown above the main content, and
	should contain anything that should be shown up there.

	The main_below sub template, conversely, is shown after the main content.
	It should probably contain the copyright statement and some other things.

	The linktree sub template should display the link tree, using the data
	in the $context['linktree'] variable.

	The menu sub template should display all the relevant buttons the user
	wants and or needs.

	For more information on the templating system, please see the site at:
	http://www.simplemachines.org/
*/

// Initialize the template... mainly little settings.
function template_init()
{
	global $context, $settings, $options, $txt;

	/* Use images from default theme when using templates from the default theme?
		if this is 'always', images from the default theme will be used.
		if this is 'defaults', images from the default theme will only be used with default templates.
		if this is 'never' or isn't set at all, images from the default theme will not be used. */
	$settings['use_default_images'] = 'never';

	/* What document type definition is being used? (for font size and other issues.)
		'xhtml' for an XHTML 1.0 document type definition.
		'html' for an HTML 4.01 document type definition. */
	$settings['doctype'] = 'xhtml';

	/* The version this template/theme is for.
		This should probably be the version of SMF it was created for. */
	$settings['theme_version'] = '2.0 RC4';

	/* Set a setting that tells the theme that it can render the tabs. */
	$settings['use_tabs'] = true;

	/* Use plain buttons - as opposed to text buttons? */
	$settings['use_buttons'] = true;

	/* Show sticky and lock status separate from topic icons? */
	$settings['separate_sticky_lock'] = true;

	/* Does this theme use the strict doctype? */
	$settings['strict_doctype'] = false;

	/* Does this theme use post previews on the message index? */
	$settings['message_index_preview'] = false;

	/* Set the following variable to true if this theme requires the optional theme strings file to be loaded. */
	$settings['require_theme_strings'] = true;
}

// The main sub template above the content.
function template_html_above()
{
	global $context, $settings, $options, $scripturl, $txt, $modSettings;

	// Show right to left and the character set for ease of translating.
	echo '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml"', $context['right_to_left'] ? ' dir="rtl"' : '', '>
<head>';

	// The ?rc3 part of this link is just here to make sure browsers don't cache it wrongly.
	echo '
	<link rel="stylesheet" type="text/css" href="', $settings['theme_url'], '/css/index', $context['theme_variant'], '.css?rc3" />';

	// Some browsers need an extra stylesheet due to bugs/compatibility issues.
	foreach (array('ie7', 'ie6', 'webkit') as $cssfix)
		if ($context['browser']['is_' . $cssfix])
			echo '
	<link rel="stylesheet" type="text/css" href="', $settings['default_theme_url'], '/css/', $cssfix, '.css" />';

	// RTL languages require an additional stylesheet.
	if ($context['right_to_left'])
		echo '
	<link rel="stylesheet" type="text/css" href="', $settings['theme_url'], '/css/rtl.css" />';

	// Here comes the JavaScript bits!
	echo '
	<script type="text/javascript" src="', $settings['default_theme_url'], '/scripts/script.js?rc3"></script>
	<script type="text/javascript" src="', $settings['theme_url'], '/scripts/theme.js?rc3"></script>
	<script type="text/javascript"><!-- // --><![CDATA[
		var smf_theme_url = "', $settings['theme_url'], '";
		var smf_default_theme_url = "', $settings['default_theme_url'], '";
		var smf_images_url = "', $settings['images_url'], '";
		var smf_scripturl = "', $scripturl, '";
		var smf_iso_case_folding = ', $context['server']['iso_case_folding'] ? 'true' : 'false', ';
		var smf_charset = "', $context['character_set'], '";', $context['show_pm_popup'] ? '
		var fPmPopup = function ()
		{
			if (confirm("' . $txt['show_personal_messages'] . '"))
				window.open(smf_prepareScriptUrl(smf_scripturl) + "action=pm");
		}
		addLoadEvent(fPmPopup);' : '', '
		var ajax_notification_text = "', $txt['ajax_in_progress'], '";
		var ajax_notification_cancel_text = "', $txt['modify_cancel'], '";
	// ]]></script>';

	echo '
	<meta http-equiv="Content-Type" content="text/html; charset=', $context['character_set'], '" />
	<meta name="description" content="', $context['page_title_html_safe'], '" />', !empty($context['meta_keywords']) ? '
	<meta name="keywords" content="' . $context['meta_keywords'] . '" />' : '', '
	<title>', $context['page_title_html_safe'], '</title>';

	// Please don't index these Mr Robot.
	if (!empty($context['robot_no_index']))
		echo '
	<meta name="robots" content="noindex" />';

	// Present a canonical url for search engines to prevent duplicate content in their indices.
	if (!empty($context['canonical_url']))
		echo '
	<link rel="canonical" href="', $context['canonical_url'], '" />';

	// Show all the relative links, such as help, search, contents, and the like.
	echo '
	<link rel="help" href="', $scripturl, '?action=help" />
	<link rel="search" href="', $scripturl, '?action=search" />
	<link rel="contents" href="', $scripturl, '" />';

	// If RSS feeds are enabled, advertise the presence of one.
	if (!empty($modSettings['xmlnews_enable']) && (!empty($modSettings['allow_guestAccess']) || $context['user']['is_logged']))
		echo '
	<link rel="alternate" type="application/rss+xml" title="', $context['forum_name_html_safe'], ' - ', $txt['rss'], '" href="', $scripturl, '?type=rss;action=.xml" />';

	// If we're viewing a topic, these should be the previous and next topics, respectively.
	if (!empty($context['current_topic']))
		echo '
	<link rel="prev" href="', $scripturl, '?topic=', $context['current_topic'], '.0;prev_next=prev" />
	<link rel="next" href="', $scripturl, '?topic=', $context['current_topic'], '.0;prev_next=next" />';

	// If we're in a board, or a topic for that matter, the index will be the board's index.
	if (!empty($context['current_board']))
		echo '
	<link rel="index" href="', $scripturl, '?board=', $context['current_board'], '.0" />';

	// Output any remaining HTML headers. (from mods, maybe?)
	echo $context['html_headers'];

	echo '
</head>
<body>';
}

function template_body_above()
{
	global $context, $settings, $options, $scripturl, $txt, $modSettings;

	echo '
		<div id="topbar">
			<div id="header" class="wrapper">
			    <div id="toplinks">
					<ul>';
						if (!empty($settings['show_date_time']))
						echo '
							<li class="time">', $context['current_time'],'</li>';

						if(!empty($settings['toplink_but1']))
						echo '
							<li><a href="', $settings['toplink_url1'] , '">' , $settings['toplink_but1'] , '</a></li>';	

						if(!empty($settings['toplink_but2']))
						echo '
							<li><a href="', $settings['toplink_url2'] , '">' , $settings['toplink_but2'] , '</a></li>';	

						if(!empty($settings['toplink_but3']))
						echo '
							<li><a href="', $settings['toplink_url3'] , '">' , $settings['toplink_but3'] , '</a></li>';

			    echo'
					</ul>
				</div>';
				if(!empty($settings['show_search']))
				{
				echo '
				<div id="searchbar">
						<form action="', $scripturl, '?action=search2" method="post" accept-charset="', $context['character_set'], '">
							<input class="inputbox" type="text" name="search" value="', $txt['forum_search'], '" onfocus="this.value = \'\';" onblur="if(this.value==\'\') this.value=\'', $txt['forum_search'], '\';" />';

					// Search within current topic?
						if (!empty($context['current_topic']))
						echo '
							<input type="hidden" name="topic" value="', $context['current_topic'], '" />';

					// If we're on a certain board, limit it to this board ;).
						elseif (!empty($context['current_board']))
						echo '
							<input type="hidden" name="brd[', $context['current_board'], ']" value="', $context['current_board'], '" />';

						echo '
						</form>
				</div>';
				}
				echo '
				<div id="logo">
					<a href="'.$scripturl.'" title=""></a>
				</div>
			</div>
		</div>
		<div id="toolbar">
			<div class="wrapper">
				' , !empty($settings['change_menustyle']) ? template_menu() : template_dropdown() , '
			</div>
		</div>
			<div class="clr"></div>
		<div id="body-bg">
			<div class="wrapper">
				<div id="userbar-shad-l">
					<div id="userbar-shad-r">
						<div id="userbar-main">
							<div id="loginbox">';
								if(!empty($context['user']['avatar']))
								echo '
									<div class="ds-avatar">'.$context['user']['avatar']['image'].'</div>';
								// If the user is logged in, display stuff like their name, new messages, etc.
								if ($context['user']['is_logged'])
								{
								echo '
										<h4>', $txt['hello_member_ndt'], ' ', $context['user']['name'],',</h4>
									<ul>';

									// Only tell them about their messages if they can read their messages!
									if ($context['allow_pm'])
									echo '
										<li>', $txt['messages'], '[', $txt['total_msg'], '<a href="', $scripturl, '?action=pm">', $context['user']['messages'], ' / <strong>', $context['user']['unread_messages'] , '</strong></a> ', $txt['pm_unread'], ']</li>';

									echo '
										<li>';
			
									// Is the forum in maintenance mode?
									if ($context['in_maintenance'] && $context['user']['is_admin'])
									echo '[<strong>', $txt['maintenance'], '</strong>]';

									// Are there any members waiting for approval?
									if (!empty($context['unapproved_members']))
									echo '[<a href="', $scripturl, '?action=admin;area=viewmembers;sa=browse;type=approve">', $context['unapproved_members'] , '', $txt['approve'], '</a>]';
																		
									echo '
										&nbsp;</li>';

									echo '
										<li><a href="', $scripturl, '?action=unread">', $txt['view_unread'], '</a> &bull;
											<a href="', $scripturl, '?action=unreadreplies">', $txt['view_replies'], '</a> &bull; 
											<a href="', $scripturl, '?action=profile;area=showposts;u=', $context['user']['id'], '">', $txt['view_own_posts'], '</a></li>';

									// Show the total time logged in?
									if (!empty($context['user']['total_time_logged_in']))
									{
									echo '
										<li>', $txt['totalTimeLogged1'];

										// If days is just zero, don't bother to show it.
										if ($context['user']['total_time_logged_in']['days'] > 0)
											echo $context['user']['total_time_logged_in']['days'] . $txt['totalTimeLogged5'];

										// Same with hours - only show it if it's above zero.
										if ($context['user']['total_time_logged_in']['hours'] > 0)
											echo $context['user']['total_time_logged_in']['hours'] . $txt['totalTimeLogged6'];

										// But, let's always show minutes - Time wasted here: 0 minutes ;).
											echo $context['user']['total_time_logged_in']['minutes'], $txt['totalTimeLogged7'], '</li>';
									}
								echo '
									</ul>';
								}

								// Otherwise they're a guest - this time ask them to either register or login - lazy bums...
								else
								{
								echo '				
									<script language="JavaScript" type="text/javascript" src="', $settings['default_theme_url'], '/scripts/sha1.js"></script>
										<form action="', $scripturl, '?action=login2" method="post" accept-charset="', $context['character_set'], '" class="smalltext" ', empty($context['disable_login_hashing']) ? ' onsubmit="hashLoginPassword(this, \'' . $context['session_id'] . '\');"' : '', '>
											<div id="ds-usernme">
												<input id="usrnme" name="user" type="text" onfocus="if(this.value==\'', $txt['username'], '\')value=\'\';" onblur="if(this.value==\'\')value=\'', $txt['username'], '\';" value="', $txt['username'], '" />
											</div>
											<div id="ds-passwrd">
												<input id="psswrd" name="passwrd" type="password" onfocus="if(this.value==\'', $txt['password'], '\')value=\'\';" onblur="if(this.value==\'\')value=\'', $txt['password'], '\';" value="', $txt['password'], '" />
											</div>
												<input id="loginbutton" type="submit" value="" name="submit" />
												<input type="hidden" name="cookielength" value="-1" />
										</form>
										<div class="reminder">
											<a href="', $scripturl, '?action=activate">', $txt['missed_act_mail'], '</a>
											<a href="', $scripturl, '?action=reminder">', $txt['forgot_pass'], '</a>
											<a href="', $scripturl, '?action=register">', $txt['create_account'], '</a>
										</div>';
								}

						echo '
							</div>
							<div id="news">';
								if (!empty($settings['enable_news']))
									echo '
										', $context['random_news_line'];

						echo '
							</div>
						</div>
					</div>
				</div>
				<div id="body-shad-r">
					<div id="body-shad-l">
						<div id="mainbody">';

	// Show the navigation tree.
	theme_linktree();
}

function template_body_below()
{
	global $context, $settings, $options, $scripturl, $txt, $modSettings;

	echo '		
						</div>
					</div>
				</div>
				<div class="clr"></div>';

			// Show the "Powered by" and "Valid" logos, as well as the copyright. Remember, the copyright must be somewhere!
			echo '
				<div id="footer">
					<div id="footer-l">
						<div id="footer-r">
							<div class="foot-float-l">
								<span class="smalltext">', theme_copyright(), '</span>';
								// Show the load time?
								if ($context['show_load_time'])
								echo '
									<br /><span class="smalltext">', $txt['page_created'], $context['load_time'], $txt['seconds_with'], $context['load_queries'], $txt['queries'], '</span>';

						echo '
							</div>
							<div class="foot-float-r">
									<span class="smalltext">DSv4 Design by <a href="http://www.dzinerstudio.com" target="_blank"><strong>DzinerStudio</strong></a></span><br />
									<a id="ds_icon" href="', $scripturl , '"></a>
							</div>';
							
					echo '
						</div>
					</div>
				</div>
			</div>
		</div>';
}

function template_html_below()
{
	global $context, $settings, $options, $scripturl, $txt, $modSettings;

	echo '
</body></html>';
}

// Show a linktree. This is that thing that shows "My Community | General Category | General Discussion"..
function theme_linktree($force_show = false)
{
	global $context, $settings, $options, $shown_linktree, $txt;

	// If linktree is empty, just return - also allow an override.
	if (empty($context['linktree']) || (!empty($context['dont_default_linktree']) && !$force_show))
		return;

	echo '
	<div class="navigate_section">
		<ul>
			<li class="ds-pathway">', $txt['you_are_here'], '</li>';

	// Each tree item has a URL and name. Some may have extra_before and extra_after.
	foreach ($context['linktree'] as $link_num => $tree)
	{
		echo '
			<li', ($link_num == count($context['linktree']) - 1) ? ' class="last"' : '', '>';

		// Show something before the link?
		if (isset($tree['extra_before']))
			echo $tree['extra_before'];

		// Show the link, including a URL if it should have one.
		echo $settings['linktree_link'] && isset($tree['url']) ? '
				<a href="' . $tree['url'] . '"><span>' . $tree['name'] . '</span></a>' : '<span>' . $tree['name'] . '</span>';

		// Show something after the link...?
		if (isset($tree['extra_after']))
			echo $tree['extra_after'];

		// Don't show a separator for the last one.
		if ($link_num != count($context['linktree']) - 1)
			echo ' &#187;';

		echo '
			</li>';
	}
	echo '
		</ul>
	</div>';

	$shown_linktree = true;
}

// Show the menu up top. Something like [home] [help] [profile] [logout]...
function template_menu()
{
	global $context, $settings, $options, $scripturl, $txt;

	echo '
		<ul id="topnav2">';

		foreach ($context['menu_buttons'] as $act => $button)
		{
			echo '
				<li' , $button['active_button'] ? ' class="chosen"' : '' , '>
					<a href="', $button['href'], '">' , $button['title'], '</a>';
			
			// any subbuttons then?
			if(!empty($button['sub_buttons']))
			{
				echo '<ul>';
				foreach ($button['sub_buttons'] as $subact => $sbutton)
					echo '
							<li>
								<a href="', $sbutton['href'], '">' , $sbutton['title'], '</a>
							</li>';
				echo '
						</ul>';
			}
			echo '
				</li>';
		}

		echo '
			</ul>';
}

// Show the menu up top. Something like [home] [help] [profile] [logout]...
function template_dropdown()
{
	global $context, $settings, $options, $scripturl, $txt;

	// Work out where we currently are.
	$current_action = 'home';
	if (in_array($context['current_action'], array('admin', 'managegames', 'arcadesettings', 'arcadecategory', 'ban', 'boardrecount', 'cleanperms', 'detailedversion', 'dumpdb', 'featuresettings', 'featuresettings2', 'findmember', 'maintain', 'manageattachments', 'manageboards', 'managecalendar', 'managesearch', 'membergroups', 'modlog', 'news', 'optimizetables', 'packageget', 'packages', 'permissions', 'pgdownload', 'postsettings', 'regcenter', 'repairboards', 'reports', 'serversettings', 'serversettings2', 'smileys', 'viewErrorLog', 'viewmembers')))
		$current_action = 'admin';
	if (in_array($context['current_action'], array('register', 'login')))
		$current_action = 'login';
	if (in_array($context['current_action'], array('search', 'admin', 'calendar', 'profile', 'mlist', 'help', 'pm', 'forum', 'unread', 'unreadreplies')))
		$current_action = 'forum';
	if ($context['current_action'] == 'search2')
		$current_action = 'search';

	if (isset($_GET['board']) || isset($_GET['topic']) || $context['current_action']=='forum')
		$current_action = 'forum';

	// Are we using right-to-left orientation?
	if ($context['right_to_left'])
	{
		$first = 'last';
		$last = 'first';
	}
	else
	{
		$first = 'first';
		$last = 'last';
	}

	// Show the start of the tab section.
	echo '	
		<ul id="topnav">
				<li ' , $current_action=='home' ? ' class="chosen"' : '' , '><a href="', $scripturl, '">' , $txt['home'] , '<br /><span>Site Front</span></a>';

				echo '<ul>
						<li><a href="', $scripturl, '?action=help">' , $txt['help'] , '</a></li>';
					if ($context['allow_search'])
					echo '
						<li><a href="', $scripturl, '?action=search">' , $txt['search'] , '</a></li>';
					if ($context['allow_calendar'])
					echo '
						<li><a href="', $scripturl, '?action=calendar">' , $txt['calendar'] , '</a></li>';
					if ($context['allow_admin'])
					echo '
						<li><a href="', $scripturl, '?action=admin">' , $txt['admin'] , '</a></li>';
					if ($context['allow_moderation_center'])
					echo '
						<li><a href="', $scripturl, '?action=moderate">' , $txt['moderate'] , '</a></li>';	
					if ($context['allow_edit_profile'])
					echo '
						<li><a href="', $scripturl, '?action=profile">' , $txt['profile'] , '</a></li>';

					if ($context['user']['is_logged'] && $context['allow_pm'])
					echo '
						<li><a href="', $scripturl, '?action=pm">' , $txt['pm_short'] , ' ', $context['user']['unread_messages'] > 0 ? '[<strong>'. $context['user']['unread_messages'] . '</strong>]' : '' , '</a></li>';

					if ($context['allow_memberlist'])
					echo '
						<li><a href="', $scripturl, '?action=mlist">' , $txt['members'] , '</a></li>';

				echo '</ul>
				</li>';
			echo '<li><a href="#">Menu Item<br /><span>Description</span></a>
					<ul>
						<li><a href="#">Submenu</a></li>
					</ul>	
				</li>';

			echo '<li><a href="#">Menu Item<br /><span>Description</span></a>
					<ul>
						<li><a href="#">Submenu</a></li>
					</ul>	
				</li>';

					if ($context['user']['is_logged'])
			echo '<li><a href="', $scripturl, '?action=logout;sesc=', $context['session_id'], '">' , $txt['logout'] , '<br /><span>Member Logout</span></a></li>';

					if ($context['user']['is_guest'])
					{
			echo '<li' , $current_action=='login' ? ' class="chosen"' : '' , '><a href="', $scripturl, '?action=login">' , $txt['login'] , '<br /><span>Member Login</span></a>';

					echo '
					<ul>
						<li' , $current_action=='register' ? ' class="chosen"' : '' , '><a href="', $scripturl, '?action=register">' , $txt['register'] , '</a></li>
					</ul>
				</li>';	
					}
	echo '
		</ul>';
}

// Generate a strip of buttons.
function template_button_strip($button_strip, $direction = 'top', $strip_options = array())
{
	global $settings, $context, $txt, $scripturl;

	if (!is_array($strip_options))
		$strip_options = array();

	// Create the buttons...
	$buttons = array();
	foreach ($button_strip as $key => $value)
	{
		if (!isset($value['test']) || !empty($context[$value['test']]))
			$buttons[] = '
				<li><a' . (isset($value['id']) ? ' id="button_strip_' . $value['id'] . '"' : '') . ' class="button_strip_' . $key . (isset($value['active']) ? ' active' : '') . '" href="' . $value['url'] . '"' . (isset($value['custom']) ? ' ' . $value['custom'] : '') . '><span>' . $txt[$value['text']] . '</span></a></li>';
	}

	// No buttons? No button strip either.
	if (empty($buttons))
		return;

	// Make the last one, as easy as possible.
	$buttons[count($buttons) - 1] = str_replace('<span>', '<span class="last">', $buttons[count($buttons) - 1]);

	echo '
		<div class="buttonlist', !empty($direction) ? ' float' . $direction : '', '"', (empty($buttons) ? ' style="display: none;"' : ''), (!empty($strip_options['id']) ? ' id="' . $strip_options['id'] . '"': ''), '>
			<ul>',
				implode('', $buttons), '
			</ul>
		</div>';
}

?>